@@ -395,7 +395,7 @@ xmlNewDoc(const CHAR *version) {
     cur->encoding = NULL;
     cur->standalone = -1;
     cur->compression = xmlCompressMode;
-#ifndef WITHOUT_CORBA
+#ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
 #endif
@@ -478,7 +478,7 @@ xmlStringLenGetNodeList(xmlDocPtr doc, const CHAR *value, int len) {
 	    while ((*cur != 0) && (cur - value < len) && (*cur != ';')) cur++;
 	    if ((*cur == 0) || (cur - value >= len)) {
 	        fprintf(stderr,
-		        "xmlStringGetNodeList: unterminated entity %30s\n", q);
+		    "xmlStringLenGetNodeList: unterminated entity %30s\n", q);
 	        return(ret);
 	    }
             if (cur != q) {
@@ -736,7 +736,7 @@ xmlNewProp(xmlNodePtr node, const CHAR *name, const CHAR *value) {
 	cur->val = xmlStringGetNodeList(node->doc, value);
     else 
 	cur->val = NULL;
-#ifndef WITHOUT_CORBA
+#ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
 #endif
@@ -792,7 +792,7 @@ xmlNewDocProp(xmlDocPtr doc, const CHAR *name, const CHAR *value) {
 	cur->val = xmlStringGetNodeList(doc, value);
     else 
 	cur->val = NULL;
-#ifndef WITHOUT_CORBA
+#ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
 #endif
@@ -879,7 +879,7 @@ xmlNewNode(xmlNsPtr ns, const CHAR *name) {
     cur->ns = ns;
     cur->nsDef = NULL;
     cur->content = NULL;
-#ifndef WITHOUT_CORBA
+#ifndef XML_WITHOUT_CORBA
     cur->_private = NULL;
     cur->vepv = NULL;
 #endif
@@ -1088,7 +1088,7 @@ xmlNewDocTextLen(xmlDocPtr doc, const CHAR *content, int len) {
  * Returns a pointer to the new node object.
  */
 xmlNodePtr
-xmlNewComment(CHAR *content) {
+xmlNewComment(const CHAR *content) {
     xmlNodePtr cur;
 
     /*
@@ -1128,7 +1128,7 @@ xmlNewComment(CHAR *content) {
  * Returns a pointer to the new node object.
  */
 xmlNodePtr
-xmlNewDocComment(xmlDocPtr doc, CHAR *content) {
+xmlNewDocComment(xmlDocPtr doc, const CHAR *content) {
     xmlNodePtr cur;
 
     cur = xmlNewComment(content);
@@ -1502,7 +1502,7 @@ xmlStaticCopyNode(xmlNodePtr node, xmlDocPtr doc, xmlNodePtr parent,
 	ret->content = xmlStrdup(node->content);
     else
 	ret->content = NULL;
-#ifndef WITHOUT_CORBA
+#ifndef XML_WITHOUT_CORBA
     ret->_private = NULL;
     ret->vepv = NULL;
 #endif
@@ -1707,6 +1707,7 @@ xmlCopyDoc(xmlDocPtr doc, int recursive) {
  * of the values carried by this node child's (TEXT and ENTITY_REF).
  * Entity references are substitued.
  * Returns a new CHAR * or NULL if no content is available.
+ *     It's up to the caller to free the memory.
  */
 CHAR *
 xmlNodeGetContent(xmlNodePtr cur) {
@@ -2106,131 +2107,285 @@ xmlTextConcat(xmlNodePtr node, const CHAR *content, int len) {
  *									*
  ************************************************************************/
 
-static CHAR *buffer = NULL;
-static int buffer_index = 0;
-static int buffer_size = 0;
+#define BASE_BUFFER_SIZE 4000
+
+/**
+ * xmlBufferCreate:
+ *
+ * routine to create an XML buffer.
+ * returns the new structure.
+ */
+xmlBufferPtr
+xmlBufferCreate(void) {
+    xmlBufferPtr ret;
+
+    ret = (xmlBufferPtr) malloc(sizeof(xmlBuffer));
+    if (ret == NULL) {
+	fprintf(stderr, "xmlBufferCreate : out of memory!\n");
+        return(NULL);
+    }
+    ret->use = 0;
+    ret->size = BASE_BUFFER_SIZE;
+    ret->content = (CHAR *) malloc(ret->size * sizeof(CHAR));
+    if (ret->content == NULL) {
+	fprintf(stderr, "xmlBufferCreate : out of memory!\n");
+	free(ret);
+        return(NULL);
+    }
+    ret->content[0] = 0;
+    return(ret);
+}
+
+/**
+ * xmlBufferFree:
+ * @buf:  the buffer to free
+ *
+ * Frees an XML buffer.
+ */
+void
+xmlBufferFree(xmlBufferPtr buf) {
+    if (buf == NULL) {
+        fprintf(stderr, "xmlBufferFree: buf == NULL\n");
+	return;
+    }
+    if (buf->content == NULL) {
+        fprintf(stderr, "xmlBufferFree: buf->content == NULL\n");
+    } else {
+        memset(buf->content, -1, BASE_BUFFER_SIZE);
+        free(buf->content);
+    }
+    memset(buf, -1, sizeof(xmlBuffer));
+    free(buf);
+}
+
+/**
+ * xmlBufferDump:
+ * @file:  the file output
+ * @buf:  the buffer to dump
+ *
+ * Dumps an XML buffer to  a FILE *.
+ * Returns the number of CHAR written
+ */
+int
+xmlBufferDump(FILE *file, xmlBufferPtr buf) {
+    int ret;
+
+    if (buf == NULL) {
+        fprintf(stderr, "xmlBufferDump: buf == NULL\n");
+	return(0);
+    }
+    if (buf->content == NULL) {
+        fprintf(stderr, "xmlBufferDump: buf->content == NULL\n");
+	return(0);
+    }
+    if (file == NULL) file = stdout;
+    ret = fwrite(buf->content, sizeof(CHAR), buf->use, file);
+    return(ret);
+}
+
+/**
+ * xmlBufferAdd:
+ * @buf:  the buffer to dump
+ * @str:  the CHAR string
+ * @len:  the number of CHAR to add
+ *
+ * Add a string range to an XML buffer.
+ */
+void
+xmlBufferAdd(xmlBufferPtr buf, const CHAR *str, int len) {
+    const CHAR *cur;
+
+    if (str == NULL) {
+        fprintf(stderr, "xmlBufferAdd: str == NULL\n");
+	return;
+    }
+    for (cur = str;(len > 0) && (*cur != 0);cur++, len--) {
+        if (buf->use  + 10 >= buf->size) {
+	    CHAR *rebuf;
+
+	    buf->size *= 2;
+	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
+	    if (rebuf == NULL) {
+	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+		return;
+	    }
+	    buf->content = rebuf;
+	}
+        buf->content[buf->use++] = *cur;
+    }
+}
+
+/**
+ * xmlBufferCat:
+ * @buf:  the buffer to dump
+ * @str:  the CHAR string
+ *
+ * Append a zero terminated string to an XML buffer.
+ */
+void
+xmlBufferCat(xmlBufferPtr buf, const CHAR *str) {
+    const CHAR *cur;
+
+    if (str == NULL) {
+        fprintf(stderr, "xmlBufferAdd: str == NULL\n");
+	return;
+    }
+    for (cur = str;*cur != 0;cur++) {
+        if (buf->use  + 10 >= buf->size) {
+	    CHAR *rebuf;
+
+	    buf->size *= 2;
+	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
+	    if (rebuf == NULL) {
+	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+		return;
+	    }
+	    buf->content = rebuf;
+	}
+        buf->content[buf->use++] = *cur;
+    }
+}
+
+/**
+ * xmlBufferCCat:
+ * @buf:  the buffer to dump
+ * @str:  the C char string
+ *
+ * Append a zero terminated C string to an XML buffer.
+ */
+void
+xmlBufferCCat(xmlBufferPtr buf, const char *str) {
+    const char *cur;
+
+    if (str == NULL) {
+        fprintf(stderr, "xmlBufferAdd: str == NULL\n");
+	return;
+    }
+    for (cur = str;*cur != 0;cur++) {
+        if (buf->use  + 10 >= buf->size) {
+	    CHAR *rebuf;
+
+	    buf->size *= 2;
+	    rebuf = (CHAR *) realloc(buf->content, buf->size * sizeof(CHAR));
+	    if (rebuf == NULL) {
+	        fprintf(stderr, "xmlBufferAdd : out of memory!\n");
+		return;
+	    }
+	    buf->content = rebuf;
+	}
+        buf->content[buf->use++] = *cur;
+    }
+}
 
 /**
  * xmlBufferWriteCHAR:
+ * @buf:  the XML buffer
  * @string:  the string to add
  *
  * routine which manage and grows an output buffer. This one add
- * CHARs at the end of the array.
+ * CHARs at the end of the buffer.
  */
 void
-xmlBufferWriteCHAR(const CHAR *string) {
-    const CHAR *cur;
-
-    if (buffer == NULL) {
-        buffer_size = 50000;
-        buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
-	if (buffer == NULL) {
-	    fprintf(stderr, "xmlBufferWrite : out of memory!\n");
-	    exit(1);
-	}
-    }
-    
-    if (string == NULL) return;
-    for (cur = string;*cur != 0;cur++) {
-        if (buffer_index  + 10 >= buffer_size) {
-	    buffer_size *= 2;
-	    buffer = (CHAR *) realloc(buffer, buffer_size * sizeof(CHAR));
-	    if (buffer == NULL) {
-	        fprintf(stderr, "xmlBufferWrite : out of memory!\n");
-		exit(1);
-	    }
-	}
-        buffer[buffer_index++] = *cur;
-    }
-    buffer[buffer_index] = 0;
+xmlBufferWriteCHAR(xmlBufferPtr buf, const CHAR *string) {
+    xmlBufferCat(buf, string);
 }
 
 /**
  * xmlBufferWriteChar:
+ * @buf:  the XML buffer output
  * @string:  the string to add
  *
  * routine which manage and grows an output buffer. This one add
  * C chars at the end of the array.
  */
 void
-xmlBufferWriteChar(const char *string) {
-    const char *cur;
-
-    if (buffer == NULL) {
-        buffer_size = 50000;
-        buffer = (CHAR *) malloc(buffer_size * sizeof(CHAR));
-	if (buffer == NULL) {
-	    fprintf(stderr, "xmlBufferWrite : out of memory!\n");
-	    exit(1);
-	}
-    }
-    
-    if (string == NULL) return;
-    for (cur = string;*cur != 0;cur++) {
-        if (buffer_index  + 10 >= buffer_size) {
-	    buffer_size *= 2;
-	    buffer = (CHAR *) realloc(buffer, buffer_size * sizeof(CHAR));
-	    if (buffer == NULL) {
-	        fprintf(stderr, "xmlBufferWrite : out of memory!\n");
-		exit(1);
-	    }
-	}
-        buffer[buffer_index++] = *cur;
-    }
-    buffer[buffer_index] = 0;
+xmlBufferWriteChar(xmlBufferPtr buf, const char *string) {
+    xmlBufferCCat(buf, string);
 }
 
+
+/**
+ * xmlBufferWriteQuotedString:
+ * @buf:  the XML buffer output
+ * @string:  the string to add
+ *
+ * routine which manage and grows an output buffer. This one writes
+ * a quoted or double quoted CHAR string, checking first if it holds
+ * quote or double-quotes internally
+ */
+void
+xmlBufferWriteQuotedString(xmlBufferPtr buf, const CHAR *string) {
+    /*
+     * TODO: fix strchr by xmlStrchr to work coreectly on UTF-8 !!!
+     */
+    if (strchr(string, '"')) {
+        if (strchr(string, '\'')) {
+	    fprintf(stderr,
+ "xmlBufferWriteQuotedString: string contains quote and double-quotes !\n");
+	}
+        xmlBufferCCat(buf, "'");
+        xmlBufferCat(buf, string);
+        xmlBufferCCat(buf, "'");
+    } else {
+        xmlBufferCCat(buf, "\"");
+        xmlBufferCat(buf, string);
+        xmlBufferCCat(buf, "\"");
+    }
+}
+
+
 /**
  * xmlGlobalNsDump:
+ * @buf:  the XML buffer output
  * @cur:  a namespace
  *
  * Dump a global Namespace, this is the old version based on PIs.
  */
 static void
-xmlGlobalNsDump(xmlNsPtr cur) {
+xmlGlobalNsDump(xmlBufferPtr buf, xmlNsPtr cur) {
     if (cur == NULL) {
         fprintf(stderr, "xmlGlobalNsDump : Ns == NULL\n");
 	return;
     }
     if (cur->type == XML_GLOBAL_NAMESPACE) {
-	xmlBufferWriteChar("<?namespace");
+	xmlBufferWriteChar(buf, "<?namespace");
 	if (cur->href != NULL) {
-	    xmlBufferWriteChar(" href=\"");
-	    xmlBufferWriteCHAR(cur->href);
-	    xmlBufferWriteChar("\"");
+	    xmlBufferWriteChar(buf, " href=");
+	    xmlBufferWriteQuotedString(buf, cur->href);
 	}
 	if (cur->prefix != NULL) {
-	    xmlBufferWriteChar(" AS=\"");
-	    xmlBufferWriteCHAR(cur->prefix);
-	    xmlBufferWriteChar("\"");
+	    xmlBufferWriteChar(buf, " AS=");
+	    xmlBufferWriteQuotedString(buf, cur->prefix);
 	}
-	xmlBufferWriteChar("?>\n");
+	xmlBufferWriteChar(buf, "?>\n");
     }
 }
 
 /**
  * xmlGlobalNsListDump:
+ * @buf:  the XML buffer output
  * @cur:  the first namespace
  *
  * Dump a list of global Namespace, this is the old version based on PIs.
  */
 static void
-xmlGlobalNsListDump(xmlNsPtr cur) {
+xmlGlobalNsListDump(xmlBufferPtr buf, xmlNsPtr cur) {
     while (cur != NULL) {
-        xmlGlobalNsDump(cur);
+        xmlGlobalNsDump(buf, cur);
 	cur = cur->next;
     }
 }
 
 /**
  * xmlNsDump:
+ * @buf:  the XML buffer output
  * @cur:  a namespace
  *
  * Dump a local Namespace definition.
  * Should be called in the context of attributes dumps.
  */
 static void
-xmlNsDump(xmlNsPtr cur) {
+xmlNsDump(xmlBufferPtr buf, xmlNsPtr cur) {
     if (cur == NULL) {
         fprintf(stderr, "xmlNsDump : Ns == NULL\n");
 	return;
@@ -2238,130 +2393,131 @@ xmlNsDump(xmlNsPtr cur) {
     if (cur->type == XML_LOCAL_NAMESPACE) {
         /* Within the context of an element attributes */
 	if (cur->prefix != NULL) {
-	    xmlBufferWriteChar(" xmlns:");
-	    xmlBufferWriteCHAR(cur->prefix);
+	    xmlBufferWriteChar(buf, " xmlns:");
+	    xmlBufferWriteCHAR(buf, cur->prefix);
 	} else
-	    xmlBufferWriteChar(" xmlns");
-	xmlBufferWriteChar("=\"");
-	xmlBufferWriteCHAR(cur->href);
-	xmlBufferWriteChar("\"");
+	    xmlBufferWriteChar(buf, " xmlns");
+	xmlBufferWriteChar(buf, "=");
+	xmlBufferWriteQuotedString(buf, cur->href);
     }
 }
 
 /**
  * xmlNsListDump:
+ * @buf:  the XML buffer output
  * @cur:  the first namespace
  *
  * Dump a list of local Namespace definitions.
  * Should be called in the context of attributes dumps.
  */
 static void
-xmlNsListDump(xmlNsPtr cur) {
+xmlNsListDump(xmlBufferPtr buf, xmlNsPtr cur) {
     while (cur != NULL) {
-        xmlNsDump(cur);
+        xmlNsDump(buf, cur);
 	cur = cur->next;
     }
 }
 
 /**
  * xmlDtdDump:
+ * @buf:  the XML buffer output
  * @doc:  the document
  * 
  * Dump the XML document DTD, if any.
  */
 static void
-xmlDtdDump(xmlDocPtr doc) {
+xmlDtdDump(xmlBufferPtr buf, xmlDocPtr doc) {
     xmlDtdPtr cur = doc->intSubset;
 
     if (cur == NULL) {
         fprintf(stderr, "xmlDtdDump : no internal subset\n");
 	return;
     }
-    xmlBufferWriteChar("<!DOCTYPE ");
-    xmlBufferWriteCHAR(cur->name);
+    xmlBufferWriteChar(buf, "<!DOCTYPE ");
+    xmlBufferWriteCHAR(buf, cur->name);
     if (cur->ExternalID != NULL) {
-	xmlBufferWriteChar(" PUBLIC \"");
-	xmlBufferWriteCHAR(cur->ExternalID);
-	xmlBufferWriteChar("\" \"");
-	xmlBufferWriteCHAR(cur->SystemID);
-	xmlBufferWriteChar("\"");
+	xmlBufferWriteChar(buf, " PUBLIC ");
+	xmlBufferWriteQuotedString(buf, cur->ExternalID);
+	xmlBufferWriteChar(buf, " ");
+	xmlBufferWriteQuotedString(buf, cur->SystemID);
     }  else if (cur->SystemID != NULL) {
-	xmlBufferWriteChar(" SYSTEM \"");
-	xmlBufferWriteCHAR(cur->SystemID);
-	xmlBufferWriteChar("\"");
+	xmlBufferWriteChar(buf, " SYSTEM ");
+	xmlBufferWriteQuotedString(buf, cur->SystemID);
     }
     if ((cur->entities == NULL) && (cur->elements == NULL) &&
         (cur->attributes == NULL) && (cur->notations == NULL)) {
-	xmlBufferWriteChar(">\n");
+	xmlBufferWriteChar(buf, ">\n");
 	return;
     }
-    xmlBufferWriteChar(" [\n");
+    xmlBufferWriteChar(buf, " [\n");
     if (cur->entities != NULL)
-	xmlDumpEntitiesTable((xmlEntitiesTablePtr) cur->entities);
+	xmlDumpEntitiesTable(buf, (xmlEntitiesTablePtr) cur->entities);
     if (cur->notations != NULL)
-	xmlDumpNotationTable((xmlNotationTablePtr) cur->notations);
+	xmlDumpNotationTable(buf, (xmlNotationTablePtr) cur->notations);
     if (cur->elements != NULL)
-	xmlDumpElementTable((xmlElementTablePtr) cur->elements);
+	xmlDumpElementTable(buf, (xmlElementTablePtr) cur->elements);
     if (cur->attributes != NULL)
-	xmlDumpAttributeTable((xmlAttributeTablePtr) cur->attributes);
-    xmlBufferWriteChar("]");
+	xmlDumpAttributeTable(buf, (xmlAttributeTablePtr) cur->attributes);
+    xmlBufferWriteChar(buf, "]");
 
     /* TODO !!! a lot more things to dump ... */
-    xmlBufferWriteChar(">\n");
+    xmlBufferWriteChar(buf, ">\n");
 }
 
 /**
  * xmlAttrDump:
+ * @buf:  the XML buffer output
  * @doc:  the document
  * @cur:  the attribute pointer
  *
  * Dump an XML attribute
  */
 static void
-xmlAttrDump(xmlDocPtr doc, xmlAttrPtr cur) {
+xmlAttrDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
     CHAR *value;
 
     if (cur == NULL) {
         fprintf(stderr, "xmlAttrDump : property == NULL\n");
 	return;
     }
-    xmlBufferWriteChar(" ");
-    xmlBufferWriteCHAR(cur->name);
+    xmlBufferWriteChar(buf, " ");
+    xmlBufferWriteCHAR(buf, cur->name);
     value = xmlNodeListGetString(doc, cur->val, 0);
     if (value) {
-	xmlBufferWriteChar("=\"");
-	xmlBufferWriteCHAR(value);
-	xmlBufferWriteChar("\"");
+	xmlBufferWriteChar(buf, "=");
+	xmlBufferWriteQuotedString(buf, value);
 	free(value);
     } else  {
-	xmlBufferWriteChar("=\"\"");
+	xmlBufferWriteChar(buf, "=\"\"");
     }
 }
 
 /**
  * xmlAttrListDump:
+ * @buf:  the XML buffer output
  * @doc:  the document
  * @cur:  the first attribute pointer
  *
  * Dump a list of XML attributes
  */
 static void
-xmlAttrListDump(xmlDocPtr doc, xmlAttrPtr cur) {
+xmlAttrListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlAttrPtr cur) {
     if (cur == NULL) {
         fprintf(stderr, "xmlAttrListDump : property == NULL\n");
 	return;
     }
     while (cur != NULL) {
-        xmlAttrDump(doc, cur);
+        xmlAttrDump(buf, doc, cur);
 	cur = cur->next;
     }
 }
 
 
 static void
-xmlNodeDump(xmlDocPtr doc, xmlNodePtr cur, int level);
+xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level);
 /**
  * xmlNodeListDump:
+ * @buf:  the XML buffer output
  * @doc:  the document
  * @cur:  the first node
  * @level: the imbrication level for indenting
@@ -2369,7 +2525,7 @@ xmlNodeDump(xmlDocPtr doc, xmlNodePtr cur, int level);
  * Dump an XML node list, recursive behaviour,children are printed too.
  */
 static void
-xmlNodeListDump(xmlDocPtr doc, xmlNodePtr cur, int level) {
+xmlNodeListDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
     int needIndent = 0, i;
 
     if (cur == NULL) {
@@ -2381,19 +2537,20 @@ xmlNodeListDump(xmlDocPtr doc, xmlNodePtr cur, int level) {
 	    (cur->type != XML_ENTITY_REF_NODE)) {
 	    if (!needIndent) {
 	        needIndent = 1;
-		xmlBufferWriteChar("\n");
+		xmlBufferWriteChar(buf, "\n");
 	    }
 	}
-        xmlNodeDump(doc, cur, level);
+        xmlNodeDump(buf, doc, cur, level);
 	cur = cur->next;
     }
     if ((xmlIndentTreeOutput) && (needIndent))
 	for (i = 1;i < level;i++)
-	    xmlBufferWriteChar("  ");
+	    xmlBufferWriteChar(buf, "  ");
 }
 
 /**
  * xmlNodeDump:
+ * @buf:  the XML buffer output
  * @doc:  the document
  * @cur:  the current node
  * @level: the imbrication level for indenting
@@ -2401,7 +2558,7 @@ xmlNodeListDump(xmlDocPtr doc, xmlNodePtr cur, int level) {
  * Dump an XML node, recursive behaviour,children are printed too.
  */
 static void
-xmlNodeDump(xmlDocPtr doc, xmlNodePtr cur, int level) {
+xmlNodeDump(xmlBufferPtr buf, xmlDocPtr doc, xmlNodePtr cur, int level) {
     int i;
 
     if (cur == NULL) {
@@ -2410,96 +2567,95 @@ xmlNodeDump(xmlDocPtr doc, xmlNodePtr cur, int level) {
     }
     if (cur->type == XML_TEXT_NODE) {
 	if (cur->content != NULL)
-	    xmlBufferWriteCHAR(xmlEncodeEntities(doc, cur->content));
+	    xmlBufferWriteCHAR(buf, xmlEncodeEntities(doc, cur->content));
 	return;
     }
     if (cur->type == XML_COMMENT_NODE) {
 	if (cur->content != NULL) {
-	    xmlBufferWriteChar("<!--");
-	    xmlBufferWriteCHAR(cur->content);
-	    xmlBufferWriteChar("-->");
+	    xmlBufferWriteChar(buf, "<!--");
+	    xmlBufferWriteCHAR(buf, cur->content);
+	    xmlBufferWriteChar(buf, "-->");
 	}
 	return;
     }
     if (cur->type == XML_ENTITY_REF_NODE) {
-        xmlBufferWriteChar("&");
-	xmlBufferWriteCHAR(cur->name);
-        xmlBufferWriteChar(";");
+        xmlBufferWriteChar(buf, "&");
+	xmlBufferWriteCHAR(buf, cur->name);
+        xmlBufferWriteChar(buf, ";");
 	return;
     }
     if (xmlIndentTreeOutput)
 	for (i = 0;i < level;i++)
-	    xmlBufferWriteChar("  ");
+	    xmlBufferWriteChar(buf, "  ");
 
-    xmlBufferWriteChar("<");
+    xmlBufferWriteChar(buf, "<");
     if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
-        xmlBufferWriteCHAR(cur->ns->prefix);
-	xmlBufferWriteChar(":");
+        xmlBufferWriteCHAR(buf, cur->ns->prefix);
+	xmlBufferWriteChar(buf, ":");
     }
 
-    xmlBufferWriteCHAR(cur->name);
+    xmlBufferWriteCHAR(buf, cur->name);
     if (cur->nsDef)
-        xmlNsListDump(cur->nsDef);
+        xmlNsListDump(buf, cur->nsDef);
     if (cur->properties != NULL)
-        xmlAttrListDump(doc, cur->properties);
+        xmlAttrListDump(buf, doc, cur->properties);
 
     if ((cur->content == NULL) && (cur->childs == NULL)) {
-        xmlBufferWriteChar("/>\n");
+        xmlBufferWriteChar(buf, "/>\n");
 	return;
     }
-    xmlBufferWriteChar(">");
+    xmlBufferWriteChar(buf, ">");
     if (cur->content != NULL)
-	xmlBufferWriteCHAR(xmlEncodeEntities(doc, cur->content));
+	xmlBufferWriteCHAR(buf, xmlEncodeEntities(doc, cur->content));
     if (cur->childs != NULL) {
-	xmlNodeListDump(doc, cur->childs, level + 1);
+	xmlNodeListDump(buf, doc, cur->childs, level + 1);
     }
-    xmlBufferWriteChar("</");
+    xmlBufferWriteChar(buf, "</");
     if ((cur->ns != NULL) && (cur->ns->prefix != NULL)) {
-        xmlBufferWriteCHAR(cur->ns->prefix);
-	xmlBufferWriteChar(":");
+        xmlBufferWriteCHAR(buf, cur->ns->prefix);
+	xmlBufferWriteChar(buf, ":");
     }
 
-    xmlBufferWriteCHAR(cur->name);
-    xmlBufferWriteChar(">\n");
+    xmlBufferWriteCHAR(buf, cur->name);
+    xmlBufferWriteChar(buf, ">\n");
 }
 
 /**
  * xmlDocContentDump:
+ * @buf:  the XML buffer output
  * @cur:  the document
  *
  * Dump an XML document.
  */
 static void
-xmlDocContentDump(xmlDocPtr cur) {
+xmlDocContentDump(xmlBufferPtr buf, xmlDocPtr cur) {
     if (oldXMLWDcompatibility)
-	xmlBufferWriteChar("<?XML version=\"");
+	xmlBufferWriteChar(buf, "<?XML version=");
     else 
-	xmlBufferWriteChar("<?xml version=\"");
-    xmlBufferWriteCHAR(cur->version);
-    xmlBufferWriteChar("\"");
+	xmlBufferWriteChar(buf, "<?xml version=");
+    xmlBufferWriteQuotedString(buf, cur->version);
     if (cur->encoding != NULL) {
-        xmlBufferWriteChar(" encoding=\"");
-	xmlBufferWriteCHAR(cur->encoding);
-	xmlBufferWriteChar("\"");
+        xmlBufferWriteChar(buf, " encoding=");
+	xmlBufferWriteQuotedString(buf, cur->encoding);
     }
     switch (cur->standalone) {
         case 0:
-	    xmlBufferWriteChar(" standalone=\"no\"");
+	    xmlBufferWriteChar(buf, " standalone=\"no\"");
 	    break;
         case 1:
-	    xmlBufferWriteChar(" standalone=\"yes\"");
+	    xmlBufferWriteChar(buf, " standalone=\"yes\"");
 	    break;
     }
-    xmlBufferWriteChar("?>\n");
+    xmlBufferWriteChar(buf, "?>\n");
     if (cur->intSubset != NULL)
-        xmlDtdDump(cur);
+        xmlDtdDump(buf, cur);
     if (cur->root != NULL) {
 	/* global namespace definitions, the old way */
 	if (oldXMLWDcompatibility)
-	    xmlGlobalNsListDump(cur->oldNs);
+	    xmlGlobalNsListDump(buf, cur->oldNs);
 	else 
 	    xmlUpgradeOldNs(cur);
-        xmlNodeDump(cur, cur->root, 0);
+        xmlNodeDump(buf, cur, cur->root, 0);
     }
 }
 
@@ -2514,6 +2670,8 @@ xmlDocContentDump(xmlDocPtr cur) {
  */
 void
 xmlDocDumpMemory(xmlDocPtr cur, CHAR**mem, int *size) {
+    xmlBufferPtr buf;
+
     if (cur == NULL) {
 #ifdef DEBUG_TREE
         fprintf(stderr, "xmlDocDumpMemory : document == NULL\n");
@@ -2522,11 +2680,17 @@ xmlDocDumpMemory(xmlDocPtr cur, CHAR**mem, int *size) {
 	*size = 0;
 	return;
     }
-    buffer_index = 0;
-    xmlDocContentDump(cur);
-
-    *mem = buffer;
-    *size = buffer_index;
+    buf = xmlBufferCreate();
+    if (buf == NULL) {
+	*mem = NULL;
+	*size = 0;
+	return;
+    }
+    xmlDocContentDump(buf, cur);
+    *mem = buf->content;
+    *size = buf->use;
+    memset(buf, -1, sizeof(xmlBuffer));
+    free(buf);
 }
 
 /**
@@ -2592,16 +2756,19 @@ xmlSetCompressMode(int mode) {
  */
 void
 xmlDocDump(FILE *f, xmlDocPtr cur) {
+    xmlBufferPtr buf;
+
     if (cur == NULL) {
 #ifdef DEBUG_TREE
         fprintf(stderr, "xmlDocDump : document == NULL\n");
 #endif
 	return;
     }
-    buffer_index = 0;
-    xmlDocContentDump(cur);
-
-    fwrite(buffer, sizeof(CHAR), buffer_index, f);
+    buf = xmlBufferCreate();
+    if (buf == NULL) return;
+    xmlDocContentDump(buf, cur);
+    xmlBufferDump(f, buf);
+    xmlBufferFree(buf);
 }
 
 /**
@@ -2615,6 +2782,7 @@ xmlDocDump(FILE *f, xmlDocPtr cur) {
  */
 int
 xmlSaveFile(const char *filename, xmlDocPtr cur) {
+    xmlBufferPtr buf;
 #ifdef HAVE_ZLIB_H
     gzFile zoutput = NULL;
     char mode[15];
@@ -2622,6 +2790,13 @@ xmlSaveFile(const char *filename, xmlDocPtr cur) {
     FILE *output = NULL;
     int ret;
 
+    /* 
+     * save the content to a temp buffer.
+     */
+    buf = xmlBufferCreate();
+    if (buf == NULL) return(0);
+    xmlDocContentDump(buf, cur);
+
 #ifdef HAVE_ZLIB_H
     if ((cur->compression > 0) && (cur->compression <= 9)) {
         sprintf(mode, "w%d", cur->compression);
@@ -2633,23 +2808,18 @@ xmlSaveFile(const char *filename, xmlDocPtr cur) {
 	if (output == NULL) return(-1);
 #ifdef HAVE_ZLIB_H
     }
-#endif
 
-    /* 
-     * save the content to a temp buffer.
-     */
-    buffer_index = 0;
-    xmlDocContentDump(cur);
-
-#ifdef HAVE_ZLIB_H
     if (zoutput != NULL) {
-        ret = gzwrite(zoutput, buffer, sizeof(CHAR) * buffer_index);
+        ret = gzwrite(zoutput, buf->content, sizeof(CHAR) * buf->use);
 	gzclose(zoutput);
-	return(ret);
+    } else {
+#endif
+        ret = xmlBufferDump(output, buf);
+	fclose(output);
+#ifdef HAVE_ZLIB_H
     }
 #endif
-    ret = fwrite(buffer, sizeof(CHAR), buffer_index, output);
-    fclose(output);
+    xmlBufferFree(buf);
     return(ret * sizeof(CHAR));
 }
 
